@@ -32,7 +32,7 @@ from nomad.datamodel.metainfo.common_dft import SingleConfigurationCalculation,
     FrameSequence
 
 from .metainfo.exciting import x_exciting_section_MT_charge_atom, x_exciting_section_MT_moment_atom,\
-    x_exciting_section_spin, x_exciting_section_xc, x_exciting_section_fermi_surface,\
+    x_exciting_section_spin, x_exciting_section_fermi_surface,\
     x_exciting_section_atoms_group
 
 
@@ -742,8 +742,7 @@ class ExcitingInfoParser(TextParser):
             'x_exciting_valence_states': ('Total number of valence states', None),
             'x_exciting_hamiltonian_size': ('Maximum Hamiltonian size', None),
             'x_exciting_pw': (r'Maximum number of plane\-waves', None),
-            'x_exciting_lo': (r'Total number of local\-orbitals', None),
-            'x_exciting_xc_functional': (r'Exchange\-correlation type', None)}
+            'x_exciting_lo': (r'Total number of local\-orbitals', None)}
 
         self._method_keys_mapping = {
             'smearing_kind': ('Smearing scheme', None),
@@ -801,6 +800,20 @@ class ExcitingInfoParser(TextParser):
             'potential_mixing', r'Using ([\w ]+) potential mixing', repeats=False, flatten=False)
         )
 
+        initialization_quantities.append(Quantity(
+            'xc_functional', r'(Exchange-correlation type[\s\S]+?\n *\n)',
+            sub_parser=TextParser(quantities=[
+                Quantity('type', r'Exchange-correlation type +: +(\S+)'),
+                Quantity(
+                    'name_reference',
+                    r'\n *(.+?,.+)',
+                    str_operation=lambda x: [v.strip() for v in x.split(':')]),
+                Quantity(
+                    'parameters',
+                    r'\n *(.+?:.+)', repeats=True,
+                    str_operation=lambda x: [v.strip() for v in x.split(':')])]))
+        )
+
         self._quantities.append(Quantity(
             'initialization',
             r'(?:All units are atomic|Starting initialization)([\s\S]+?)(?:Using|Ending initialization)', repeats=False,
@@ -851,32 +864,34 @@ class ExcitingInfoParser(TextParser):
                 name, r'%s\s*\:*\s*([\(\)\d\.\-\+Ee ]+)' % key_unit[0],
                 str_operation=str_to_quantity_tolerances, unit=key_unit[1], repeats=False))
 
+        module_quantities = [
+            Quantity(
+                'scf_iteration', r'(?:I| i)teration number :([\s\S]+?)(?:\n *\n\+{10}|\+\-{10})',
+                sub_parser=TextParser(quantities=scf_quantities), repeats=True),
+            Quantity(
+                'final',
+                r'(?:Convergence targets achieved\. Performing final SCF iteration|Reached self-consistent loops maximum)([\s\S]+?)(\n *\n\+{10})',
+                sub_parser=TextParser(quantities=scf_quantities), repeats=False),
+            Quantity(
+                'atomic_positions',
+                r'(Atomic positions\s*\([\s\S]+?)\n\n',
+                sub_parser=TextParser(quantities=[
+                    Quantity(
+                        'positions_format', r'Atomic positions\s*\(([a-z]+)\)'),
+                    Quantity(
+                        'symbols', r'atom\s*\d+\s*(\w+)', repeats=True, dtype=str),
+                    Quantity(
+                        'positions', r'\s*:\s*([\d\.\-]+\s*[\d\.\-]+\s*[\d\.\-]+)',
+                        repeats=True, dtype=float)])),
+            Quantity(
+                'forces', r'Total atomic forces including IBS \(\w+\)\s*\:(\s*atom[\-\s\w\.\:]*?)\n *Atomic',
+                repeats=False, str_operation=str_to_array, dtype=float, unit=ureg.hartree / ureg.bohr)
+        ]
+
         self._quantities.append(Quantity(
             'groundstate',
             r'(?:Self\-consistent loop started|Groundstate module started)([\s\S]+?)Groundstate module stopped',
-            sub_parser=TextParser(quantities=[
-                Quantity(
-                    'scf_iteration', r'(?:I|SCF i)teration number :([\s\S]+?)(?:\n *\n\+{10}|\+\-{10})',
-                    sub_parser=TextParser(quantities=scf_quantities), repeats=True),
-                Quantity(
-                    'final',
-                    r'(?:Convergence targets achieved\. Performing final SCF iteration|Reached self-consistent loops maximum)([\s\S]+?)(\n *\n\+{10})',
-                    sub_parser=TextParser(quantities=scf_quantities), repeats=False),
-                Quantity(
-                    'atomic_positions',
-                    r'(Atomic positions\s*\([\s\S]+?)\n\n',
-                    sub_parser=TextParser(quantities=[
-                        Quantity(
-                            'positions_format', r'Atomic positions\s*\(([a-z]+)\)'),
-                        Quantity(
-                            'symbols', r'atom\s*\d+\s*(\w+)', repeats=True, dtype=str),
-                        Quantity(
-                            'positions', r'\s*:\s*([\d\.\-]+\s*[\d\.\-]+\s*[\d\.\-]+)',
-                            repeats=True, dtype=float)])),
-                Quantity(
-                    'forces', r'Total atomic forces including IBS \(\w+\)\s*\:(\s*atom[\-\s\w\.\:]*?)\n *Atomic',
-                    repeats=False, str_operation=str_to_array, dtype=float, unit=ureg.hartree / ureg.bohr)
-            ]), repeats=False))
+            sub_parser=TextParser(quantities=module_quantities), repeats=False))
 
         optimization_quantities = [
             Quantity(
@@ -945,6 +960,11 @@ class ExcitingInfoParser(TextParser):
                     repeats=False, str_operation=str_to_array, dtype=float, unit=ureg.hartree / ureg.bohr),
             ]), repeats=False))
 
+        self._quantities.append(Quantity(
+            'hybrids',
+            r'Hybrids module started([\s\S]+?)Hybrids module stopped',
+            sub_parser=TextParser(quantities=module_quantities)))
+
     def get_atom_labels(self, section):
         labels = section.get('symbols')
 
@@ -991,7 +1011,8 @@ class ExcitingInfoParser(TextParser):
         return positions * ureg.bohr
 
     def get_scf_threshold(self, name):
-        return self.get('groundstate', {}).get('scf_iteration', [{}])[-1].get(
+        reference = self.get('groundstate', self.get('hybrids', {}))
+        return reference.get('scf_iteration', [{}])[-1].get(
             name, [None, None])[-1]
 
     def get_scf_quantity(self, name):
@@ -1021,13 +1042,14 @@ class ExcitingInfoParser(TextParser):
             26: ['GGA_C_PBE', 'GGA_X_WC'],
             30: ['GGA_C_AM05', 'GGA_C_AM05'],
             300: ['GGA_C_BGCP', 'GGA_X_PBE'],
-            406: ['HYB_GGA_XC_PBEH']}
+            406: ['HYB_GGA_XC_PBEH'],
+            408: ['HYB_GGA_XC_HSE03']}
 
-        xc_functional = self.get('initialization', {}).get('x_exciting_xc_functional', None)
+        xc_functional = self.get('initialization', {}).get('xc_functional', None)
         if xc_functional is None:
             return []
 
-        name = xc_functional_map.get(xc_functional, [])
+        name = xc_functional_map.get(xc_functional.type, [])
 
         return name
 
@@ -1289,6 +1311,8 @@ class ExcitingParser(FairdiParser):
         nspin = self.info_parser.get_number_of_spin_channels()
 
         def reshape(data):
+            if data[0] is None:
+                return
             return np.reshape(data, (nspin, len(data) // nspin, len(data[0])))
 
         sec_eigenvalues.number_of_eigenvalues = len(eigs_gw[0])
@@ -2106,9 +2130,6 @@ class ExcitingParser(FairdiParser):
             if name == 'x_exciting_spin_treatment':
                 sub_sec = sec_system.m_create(x_exciting_section_spin)
                 sub_sec.x_exciting_spin_treatment = val
-            elif name == 'x_exciting_xc_functional':
-                sub_sec = sec_system.m_create(x_exciting_section_xc)
-                sub_sec.x_exciting_xc_functional = val
             elif name == 'x_exciting_species_rtmin':
                 setattr(sec_system, name, ' '.join([str(v) for v in val]))
             else:
@@ -2168,8 +2189,10 @@ class ExcitingParser(FairdiParser):
             return sec_scc
 
         # groundstate calculation
-        sec_scc = parse_configuration(self.info_parser.get('groundstate'))
-        if sec_scc is not None:
+        for module in ['groundstate', 'hybrids']:
+            sec_scc = parse_configuration(self.info_parser.get(module))
+            if sec_scc is None:
+                continue
             # add data to scc
             # TODO add support for more output files and properties
             exciting_files = ['EIGVAL.OUT', 'FERMISURF.bxsf', 'FS.bxsf']
