@@ -16,4 +16,4 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-from libatomsparser.libAtomsParser import LibAtomsParserWrapper
+from .libatoms_parser import LibAtomsParser
