@@ -508,11 +508,15 @@ def edit(parsed_query: Dict[str, Any], mongo_update: Dict[str, Any] = None, re_i
                         upload_files = files.UploadFiles.get(upload_id, is_authorized=lambda: True)
                         upload_files_cache[upload_id] = upload_files
 
-                    entry_metadata = calc.entry_metadata(upload_files)
-                    entry = entry_metadata.a_elastic.create_index_entry().to_dict(include_meta=True)
-                    entry['_op_type'] = 'index'
+                    try:
+                        entry_metadata = calc.entry_metadata(upload_files)
+                        entry = entry_metadata.a_elastic.create_index_entry().to_dict(include_meta=True)
+                        entry['_op_type'] = 'index'
 
-                    yield entry
+                        yield entry
+
+                    except Exception as e:
+                        common.logger.error('edit repo could not create index doc', exc_info=e)
 
                 for upload_files in upload_files_cache.values():
                     upload_files.close()
@@ -663,6 +667,24 @@ class EditRepoCalcsResource(Resource):
                     else:
                         mongo_value = action_value
 
+                    # verify if the edit action creates consistent shared_with and with_embargo for
+                    # whole uploads
+                    if action_quantity_name == 'shared_with' or action_quantity_name == 'with_embargo':
+                        search_request = search.SearchRequest()
+                        apply_search_parameters(search_request, parsed_query)
+                        search_request.quantity('upload_id')
+                        uploads = search_request.execute()['quantities']['upload_id']['values']
+                        for upload_id, upload_data in uploads.items():
+                            search_request = search.SearchRequest().search_parameters(upload_id=upload_id)
+                            entries = search_request.execute()['total']
+                            if entries != upload_data['total']:
+                                action['success'] = False
+                                action['message'] = json_data.get('message', '') + (
+                                    'Edit would create an upload with inconsistent shared_with or with_embargo. '
+                                    'You can only set those for all entries of an upload.')
+                                has_error = True
+                                continue
+
                     if len(quantity.shape) == 0:
                         mongo_update[mongo_key] = mongo_value
                     else:
@@ -715,7 +737,8 @@ class EditRepoCalcsResource(Resource):
         if lift_embargo:
             for upload_id in upload_ids:
                 upload = proc.Upload.get(upload_id)
-                upload.re_pack()
+                if upload.published:
+                    upload.re_pack()
 
         # remove potentially empty old datasets
         if removed_datasets is not None:
