@@ -31,7 +31,7 @@ from nomad.utils import strip
 from nomad.metainfo import Datetime, MEnum
 from nomad.metainfo.search_extension import metrics, search_quantities, search_sub_sections
 
-from .utils import parameter_dependency_from_model
+from .utils import parameter_dependency_from_model, update_url_query_arguments
 
 
 User = datamodel.User.m_def.a_pydantic.model
@@ -438,34 +438,209 @@ metadata_required_parameters = parameter_dependency_from_model(
 class Pagination(BaseModel):
     ''' Defines the order, size, and page of results. '''
 
-    size: Optional[int] = Field(
+    page_size: Optional[int] = Field(
         10, description=strip('''
             The page size, e.g. the maximum number of items contained in one response.
-            A `size` of 0 will omit any results.
+            A `page_size` of 0 will return no results.
         '''))
     order_by: Optional[str] = Field(
-        calc_id,  # type: ignore
+        None,  # type: ignore
         description=strip('''
-            The results are ordered by the values of this field. The response
-            either contains the first `size` value or the next `size` values after `after`.
+            The results are ordered by the values of this field. If omitted, default
+            ordering is applied.
         '''))
     order: Optional[Direction] = Field(
         Direction.asc, description=strip('''
-            The order direction of the results based on `order_by`. Its either
-            ascending `asc` or decending `desc`.
+            The ordering direction of the results based on `order_by`. Its either
+            ascending `asc` or decending `desc`. Default is `asc`.
         '''))
-    after: Optional[str] = Field(
+    page_after_value: Optional[str] = Field(
         None, description=strip('''
-            A request for the page after this value, i.e. the next `size` values behind `after`.
-            This depends on the `order_by`.
-            Each response contains the `after` value for the *next* request following
-            the defined order.
+            This attribute defines the position after which the page begins, and is used
+            to navigate through the total list of results.
 
-            The after value and its type depends on the API operation and potentially on
-            the `order_by` field and its type.
-            The after value will always be a string encoded value. It might be an `order_by` value, or an index.
-            The after value might contain an id as *tie breaker*, if `order_by` is not the unique.
-            The *tie breaker* will be `:` separated, e.g. `<value>:<id>`.
+            When requesting the first page, no value should be provided for
+            `page_after_value`. Each response will contain a value `next_page_after_value`,
+            which can be used to obtain the next page (by setting `page_after_value` in
+            your next request to this value).
+
+            The field is encoded as a string, and the format of `page_after_value` and
+            `next_page_after_value` depends on which API method is used.
+
+            Some API functions additionally allows a simplified navigation, by specifying
+            the page number in the key `page`. It is however always possible to use
+            `page_after_value` and `next_page_after_value` to iterate through the results.
+            '''))
+    page: Optional[int] = Field(
+        None, description=strip('''
+            The number of the page (1-based). When provided in a request, this attribute
+            can be used instead of `page_after_value` to jump to a particular results page.
+
+            **NOTE #1**: the option to request pages by submitting the `page` number is
+            limited. There are api calls where this attribute cannot be used for indexing,
+            or where it can only be used partially. **If you want to just iterate through
+            all the results, aways use the `page_after_value` and `next_page_after_value`!**
+
+            **NOTE #2**: In a request, you should never specify *both* `page` and
+            `page_after_value` (at most one index can be provided).
+        '''))
+
+    @validator('page_size')
+    def validate_page_size(cls, page_size):  # pylint: disable=no-self-argument
+        assert page_size >= 0, 'page_size must be >= 0'
+        return page_size
+
+    @validator('order_by')
+    def validate_order_by(cls, order_by):  # pylint: disable=no-self-argument
+        '''
+        Override this in your Pagination class to ensure that a valid attribute is selected.
+        This method has to be implemented!
+        '''
+        raise NotImplementedError('Validation of `order_by` not implemented!')
+
+    @validator('page_after_value')
+    def validate_page_after_value(cls, page_after_value, values):  # pylint: disable=no-self-argument
+        '''
+        Override this in your Pagination class to implement validation of the
+        `page_after_value` value.
+        This method has to be implemented!
+        '''
+        raise NotImplementedError('Validation of `page_after_value` not implemented!')
+
+    @validator('page')
+    def validate_page(cls, page, values):  # pylint: disable=no-self-argument
+        if page is not None:
+            assert page >= 1, 'page must be >= 1'
+        return page
+
+    @root_validator(skip_on_failure=True)
+    def validate_values(cls, values):  # pylint: disable=no-self-argument
+        # Because of a bug in pydantic (#2670), root validators can't be overridden, so
+        # we invoke a class method, which *can* be overridden.
+        return cls._root_validation(values)
+
+    @classmethod
+    def _root_validation(cls, values):
+        page = values.get('page')
+        page_after_value = values.get('page_after_value')
+        page_size = values.get('page_size')
+        assert page is None or page_after_value is None, 'Cannot specify both `page` and `page_after_value'
+        if page_size == 0:
+            assert page is None, 'Cannot specify `page` when `page_size` is set to 0'
+            assert page_after_value is None, 'Cannot specify `page_after_value` when `page_size` is set to 0'
+        return values
+
+    def get_simple_index(self):
+        '''
+        If simple, index-based pagination is used, this method can be used to get the
+        corresponding index (0-based). It will look on either `page` or `page_after_value`.
+        If neither index is provided, we return 0 (i.e. the first index).
+        '''
+        if self.page is None and self.page_after_value is None:
+            return 0
+        if self.page is not None:
+            return (self.page - 1) * self.page_size
+        rv = int(self.page_after_value) + 1
+        assert rv >= 0
+        return rv
+
+
+class PaginationResponse(Pagination):
+    total: int = Field(
+        ..., description=strip('''
+        The total number of results that fit the given query. This is independent of
+        any pagination and aggregations.
+        '''))
+    next_page_after_value: Optional[str] = Field(
+        None, description=strip('''
+        The *next* value to be used as `page_after_value` in a follow up requests, to get
+        the next page of results. If no more results are available, `next_page_after_value`
+        will not be set.
+        '''))
+    page_url: Optional[str] = Field(
+        None, description=strip('''
+        The url of the current page. Only applicable for GET requests.
+        '''))
+    next_page_url: Optional[str] = Field(
+        None, description=strip('''
+        The url to get the next page. Only applicable for GET requests.
+        '''))
+    prev_page_url: Optional[str] = Field(
+        None, description=strip('''
+        The url to get the previous page. **NOTE:** Only applicable for some API methods,
+        (namely, where indexing by `page` is possible), and only for GET requests.
+        '''))
+    first_page_url: Optional[str] = Field(
+        None, description=strip('''
+        The url to get the first page. Only applicable for GET requests.
+        '''))
+
+    @validator('order_by')
+    def validate_order_by(cls, order_by):  # pylint: disable=no-self-argument
+        # No validation - behaviour of this field depends on api method
+        return order_by
+
+    @validator('page_after_value')
+    def validate_page_after_value(cls, page_after_value, values):  # pylint: disable=no-self-argument
+        # No validation - behaviour of this field depends on api method
+        return page_after_value
+
+    @classmethod
+    def _root_validation(cls, values):  # pylint: disable=no-self-argument
+        # No validation
+        return values
+
+    def populate_urls(self, request: Request):
+        '''
+        Populates the urls (`page_url`, `next_page_url`, `first_page_url` from the
+        request and `next_page_after_value`. Only applicable for GET requests.
+        '''
+        assert request.method.upper() == 'GET', 'Trying to populate urls, but method is not GET.'
+        original_url = str(request.url)
+        self.page_url = original_url
+        if self.page_size:
+            self.first_page_url = update_url_query_arguments(
+                original_url, page=None, page_after_value=None)
+        if self.next_page_after_value:
+            self.next_page_url = update_url_query_arguments(
+                original_url, page=None, page_after_value=self.next_page_after_value)
+        if self.page and self.page > 1:
+            self.prev_page_url = update_url_query_arguments(
+                original_url, page=self.page - 1, page_after_value=None)
+
+    def populate_simple_index_and_urls(self, request: Request):
+        '''
+        If simple, index-based pagination is used, this method can be used to populate
+        the `page`, `page_after_value` and urls (if it is a GET request) automatically.
+        Assumes that the field `total` is populated.
+        '''
+        if not self.page_size:
+            self.page = 1
+            self.page_after_value = None
+            self.next_page_after_value = None
+        else:
+            ind = self.get_simple_index()
+            self.page = ind // self.page_size + 1
+            self.page_after_value = None if self.page == 1 else str(ind - 1)
+            if self.page_size * self.page >= self.total:
+                self.next_page_after_value = None
+            else:
+                self.next_page_after_value = str(ind + self.page_size - 1)
+
+            if self.page < 1 or (
+                    self.total == 0 and self.page != 1) or (
+                    self.total > 0 and (self.page - 1) * self.page_size >= self.total):
+                raise HTTPException(400, detail='Page out of range requested.')
+        if request.method.upper() == 'GET':
+            self.populate_urls(request)
+
+
+class EntryBasedPagination(Pagination):
+    order_by: Optional[str] = Field(
+        calc_id,  # type: ignore
+        description=strip('''
+            The results are ordered by the values of this field. If omitted, default
+            ordering is applied.
         '''))
 
     @validator('order_by')
@@ -478,30 +653,52 @@ class Pagination(BaseModel):
         assert quantity.definition.is_scalar, 'the order_by quantity must be a scalar'
         return order_by
 
-    @validator('size')
-    def validate_size(cls, size):  # pylint: disable=no-self-argument
-        assert size >= 0, 'size must be positive integer'
-        return size
-
-    @validator('after')
-    def validate_after(cls, after, values):  # pylint: disable=no-self-argument
+    @validator('page_after_value')
+    def validate_page_after_value(cls, page_after_value, values):  # pylint: disable=no-self-argument
         order_by = values.get('order_by', calc_id)
-        if after is not None and order_by is not None and order_by != calc_id and ':' not in after:
-            after = '%s:' % after
-        return after
+        if page_after_value is not None and order_by is not None and order_by != calc_id:
+            if ':' not in page_after_value:
+                page_after_value = '%s:' % page_after_value
+        return page_after_value
 
 
-pagination_parameters = parameter_dependency_from_model(
-    'pagination_parameters', Pagination)
-
-
-class AggregationPagination(Pagination):
-    order_by: Optional[str] = Field(
+class EntryPagination(EntryBasedPagination):
+    page: Optional[int] = Field(
         None, description=strip('''
-        The search results are ordered by the values of this quantity. The response
-        either contains the first `size` value or the next `size` values after `after`.
+            For simple, index-based pagination, this should contain the number of the
+            requested page (1-based). When provided in a request, this attribute can be
+            used instead of `page_after_value` to jump to a particular results page.
+
+            However, you can only retreive up to the 10.000th entry with a page number.
+            Only one, `page_after_value` *or* `page` can be provided.
         '''))
 
+    @validator('page')
+    def validate_page(cls, page, values):  # pylint: disable=no-self-argument
+        if page is not None:
+            assert page > 0, 'Page has to be larger than 1.'
+            assert page * values.get('page_size', 10) < 10000, 'Pagination by `page` is limited to 10.000 entries.'
+
+        return page
+
+
+entry_pagination_parameters = parameter_dependency_from_model(
+    'entry_pagination_parameters', EntryPagination)
+
+
+class AggregationPagination(EntryBasedPagination):
+    order_by: Optional[str] = Field(
+        None,  # type: ignore
+        description=strip('''
+            The results are ordered by the values of this field. If omitted, default
+            ordering is applied.
+        '''))
+
+    @validator('page')
+    def validate_page(cls, page, values):  # pylint: disable=no-self-argument
+        assert page is None, 'Pagination by `page` is not possible for aggregations, use `page_after_value`'
+        return page
+
 
 class AggregatedEntities(BaseModel):
     size: Optional[pydantic.conint(gt=0)] = Field(  # type: ignore
@@ -590,10 +787,10 @@ class Statistic(BaseModel):
 
 
 class WithQueryAndPagination(WithQuery):
-    pagination: Optional[Pagination] = Body(
+    pagination: Optional[EntryPagination] = Body(
         None,
         example={
-            'size': 5,
+            'page_size': 5,
             'order_by': 'upload_time'
         })
 
@@ -625,7 +822,7 @@ class EntriesMetadata(WithQueryAndPagination):
             'uploads': {
                 'quantity': 'upload_id',
                 'pagination': {
-                    'size': 10,
+                    'page_size': 10,
                     'order_by': 'upload_time'
                 },
                 'entries': {
@@ -698,74 +895,108 @@ files_parameters = parameter_dependency_from_model(
 ArchiveRequired = Union[str, Dict[str, Any]]
 
 
-class EntriesArchive(WithQueryAndPagination):
-    required: Optional[ArchiveRequired] = Body(
-        '*',
-        embed=True,
-        description=strip('''
-            The `required` part allows you to specify what parts of the requested archives
-            should be returned. The NOMAD Archive is a hierarchical data format and
-            you can *require* certain branches (i.e. *sections*) in the hierarchy.
-            By specifing certain sections with specific contents or all contents (via `"*"`),
-            you can determine what sections and what quantities should be returned.
-            The default is everything: `"*"`.
+_archive_required_field = Body(
+    '*',
+    embed=True,
+    description=strip('''
+        The `required` part allows you to specify what parts of the requested archives
+        should be returned. The NOMAD Archive is a hierarchical data format and
+        you can *require* certain branches (i.e. *sections*) in the hierarchy.
+        By specifing certain sections with specific contents or all contents (via
+        the directive `"*"`), you can determine what sections and what quantities should
+        be returned. The default is the whole archive, i.e., `"*"`.
 
-            For example to specify that you are only interested in the `section_metadata`
-            use:
+        For example to specify that you are only interested in the `section_metadata`
+        use:
 
-            ```
-            {
-                "section_run": "*"
+        ```
+        {
+            "section_metadata": "*"
+        }
+        ```
+
+        Or to only get the `energy_total` from each individual calculations, use:
+        ```
+        {
+            "section_run": {
+                "section_single_configuration_calculation": {
+                    "energy_total": "*"
+                }
             }
-            ```
+        }
+        ```
 
-            Or to only get the `energy_total` from each individual calculations, use:
-            ```
-            {
-                "section_run": {
-                    "section_single_configuration_calculation": {
-                        "energy_total": "*"
+        You can also request certain parts of a list, e.g. the last calculation:
+        ```
+        {
+            "section_run": {
+                "section_single_configuration_calculation[-1]": "*"
+            }
+        }
+        ```
+
+        These required specifications are also very useful to get workflow results.
+        This works because we can use references (e.g. workflow to final result calculation)
+        and the API will resolve these references and return the respective data.
+        For example just the total energy value and reduced formula from the resulting
+        calculation:
+        ```
+        {
+            "section_workflow": {
+                "calculation_result_ref": {
+                    "energy_total": "*",
+                    "single_configuration_calculation_to_system_ref": {
+                        "chemical_composition_reduced": "*"
                     }
                 }
             }
-            ```
+        }
+        ```
 
-            You can also request certain parts of a list, e.g. the last calculation:
-            ```
-            {
-                "section_run": {
-                    "section_single_configuration_calculation[-1]": "*"
-                }
+        You can also resolve all references in a branch with the `include-resolved`
+        directive. This will resolve all references in the branch, and also all references
+        in referenced sections:
+        ```
+        {
+            "section_workflow":
+                "calculation_result_ref": "include-resolved"
             }
-            ```
+        }
+        ```
 
-            These required specifications are also very useful to get workflow results.
-            This works because we can use references (e.g. workflow to final result calculation)
-            and the API will resolve these references and return the respective data.
-            For example just the total energy value and reduced formula from the resulting
-            calculation:
-            ```
-            {
-                'section_workflow': {
-                    'calculation_result_ref': {
-                        'energy_total': '*',
-                        'single_configuration_calculation_to_system_ref': {
-                            'chemical_composition_reduced': '*'
-                        }
-                    }
-                }
+        By default, the targets of "resolved" references are added to the archive at
+        their original hierarchy positions.
+        This means, all references are still references, but they are resolvable within
+        the returned data, since they targets are now part of the data. Another option
+        is to add
+        `"resolve-inplace": true` to the root of required. Here, the reference targets will
+        replace the references:
+        ```
+        {
+            "resolve-inplace": true,
+            "section_workflow":
+                "calculation_result_ref": "include-resolved"
             }
-            ```
-        '''),
-        example={
-            'section_run': {
-                'section_single_configuration_calculation[-1]': {
-                    'energy_total': '*'
-                },
-                'section_system[-1]': '*'
+        }
+        ```
+    '''),
+    example={
+        'section_run': {
+            'section_single_configuration_calculation[-1]': {
+                'energy_total': '*'
             },
-            'section_metadata': '*'
-        })
+            'section_system[-1]': '*'
+        },
+        'section_metadata': '*'
+    })
+
+
+class EntriesArchive(WithQueryAndPagination):
+    required: Optional[ArchiveRequired] = _archive_required_field
+
+
+class EntryArchiveRequest(BaseModel):
+    required: Optional[ArchiveRequired] = _archive_required_field
 
 
 class EntriesArchiveDownload(WithQuery):
@@ -773,7 +1004,7 @@ class EntriesArchiveDownload(WithQuery):
 
 
 class EntriesRaw(WithQuery):
-    pagination: Optional[Pagination] = Body(None)
+    pagination: Optional[EntryPagination] = Body(None)
 
 
 class EntriesRawDownload(WithQuery):
@@ -784,17 +1015,6 @@ class EntriesRawDownload(WithQuery):
         })
 
 
-class PaginationResponse(Pagination):
-    total: int = Field(..., description=strip('''
-        The total number of entries that fit the given `query`. This is independent of
-        any pagination and aggregations.
-    '''))
-    next_after: Optional[str] = Field(None, description=strip('''
-        The *next* after value to be used as `after` in a follow up requests for the
-        next page of results.
-    '''))
-
-
 class StatisticResponse(Statistic):
     data: Dict[str, Dict[str, int]] = Field(
         None, description=strip('''
@@ -828,7 +1048,7 @@ class CodeResponse(BaseModel):
 
 
 class EntriesMetadataResponse(EntriesMetadata):
-    pagination: PaginationResponse
+    pagination: PaginationResponse  # type: ignore
     statistics: Optional[Dict[str, StatisticResponse]]  # type: ignore
     aggregations: Optional[Dict[str, AggregationResponse]]  # type: ignore
     data: List[Dict[str, Any]] = Field(
@@ -851,7 +1071,7 @@ class EntryRaw(BaseModel):
 
 
 class EntriesRawResponse(EntriesRaw):
-    pagination: PaginationResponse = Field(None)
+    pagination: PaginationResponse = Field(None)  # type: ignore
     data: List[EntryRaw] = Field(None)
 
 
@@ -871,17 +1091,17 @@ class EntryArchive(BaseModel):
     calc_id: str = Field(None)
     upload_id: str = Field(None)
     parser_name: str = Field(None)
-    archive: Any = Field(None)
+    archive: Dict[str, Any] = Field(None)
 
 
 class EntriesArchiveResponse(EntriesArchive):
-    pagination: PaginationResponse = Field(None)
+    pagination: PaginationResponse = Field(None)  # type: ignore
     data: List[EntryArchive] = Field(None)
 
 
-class EntryArchiveResponse(BaseModel):
+class EntryArchiveResponse(EntryArchiveRequest):
     entry_id: str = Field(...)
-    data: Dict[str, Any]
+    data: EntryArchive = Field(None)
 
 
 class SearchResponse(EntriesMetadataResponse):
