@@ -16,6 +16,7 @@
 # limitations under the License.
 #
 
+from subprocess import PIPE
 import typing
 import click
 import tabulate
@@ -192,6 +193,49 @@ def ls(ctx, uploads, calculations, ids, json):
         headers=headers))
 
 
+@uploads.command(help=(
+    'Allows to edit metadata attribute of all entries in uploads. Be aware that this '
+    'only edits the attributes. E.g. if you set publish true, it won\'t publish the '
+    'upload, pack its files, change the upload metadata, etc.'))
+@click.option(
+    '--publish', type=click.Choice(['with-embargo', 'no-embargo']),
+    help='Set the publish attribute true and change with_embargo attribute.')
+@click.option(
+    '--unpublish', is_flag=True, help='Set the publish attribute to false.')
+@click.argument('UPLOADS', nargs=-1)
+@click.pass_context
+def edit(ctx, uploads, publish: str, unpublish: bool):
+    _, uploads = query_uploads(ctx, uploads)
+
+    if publish and unpublish:
+        print('You can only publish or unpublish, not both.')
+        return
+
+    if publish:
+        update = {
+            'metadata.published': True,
+            'metadata.with_embargo': publish == 'with-embargo'}
+    elif unpublish:
+        update = {
+            'metadata.published': False,
+            'metadata.with_embargo': False}
+    else:
+        print('You have not give any attributes to edit.')
+        return
+
+    print('%d uploads selected, editing ...' % uploads.count())
+
+    for upload in uploads:
+        proc.Calc._get_collection().update_many(
+            {'upload_id': upload.upload_id},
+            {'$set': update})
+
+        with upload.entries_metadata() as calcs:
+            search.index_all(calcs, do_refresh=False)
+
+    search.refresh()
+
+
 @uploads.command(help='Change the owner of the upload and all its calcs.')
 @click.argument('USERNAME', nargs=1)
 @click.argument('UPLOADS', nargs=-1)
@@ -224,8 +268,10 @@ def chown(ctx, username, uploads):
 @uploads.command(help='Reset the processing state.')
 @click.argument('UPLOADS', nargs=-1)
 @click.option('--with-calcs', is_flag=True, help='Also reset all calculations.')
+@click.option('--success', is_flag=True, help='Set the tasks status to success instead of pending')
+@click.option('--failure', is_flag=True, help='Set the tasks status to failure instead of pending.')
 @click.pass_context
-def reset(ctx, uploads, with_calcs):
+def reset(ctx, uploads, with_calcs, success, failure):
     _, uploads = query_uploads(ctx, uploads)
     uploads_count = uploads.count()
 
@@ -233,12 +279,22 @@ def reset(ctx, uploads, with_calcs):
 
     i = 0
     for upload in uploads:
-        proc.Calc._get_collection().update_many(
-            dict(upload_id=upload.upload_id),
-            {'$set': proc.Calc.reset_pymongo_update()})
+        if with_calcs:
+            calc_update = proc.Calc.reset_pymongo_update()
+            if success:
+                calc_update['tasks_status'] = proc.SUCCESS
+            if failure:
+                calc_update['tasks_status'] = proc.FAILURE
+
+            proc.Calc._get_collection().update_many(
+                dict(upload_id=upload.upload_id), {'$set': calc_update})
 
         upload.process_status = None
         upload.reset()
+        if success:
+            upload.tasks_status = proc.SUCCESS
+        if failure:
+            upload.tasks_status = proc.FAILURE
         upload.save()
         i += 1
         print('resetted %d of %d uploads' % (i, uploads_count))
@@ -331,7 +387,7 @@ def re_process(ctx, uploads, parallel: int, reprocess_running: bool):
     _, uploads = query_uploads(ctx, uploads)
     __run_processing(
         uploads, parallel, lambda upload: upload.re_process_upload(), 're-processing',
-        reprocess_running=reprocess_running)
+        reprocess_running=reprocess_running, reset_first=True)
 
 
 @uploads.command(help='Repack selected uploads.')
@@ -340,7 +396,145 @@ def re_process(ctx, uploads, parallel: int, reprocess_running: bool):
 @click.pass_context
 def re_pack(ctx, uploads, parallel: int):
     _, uploads = query_uploads(ctx, uploads)
-    __run_processing(uploads, parallel, lambda upload: upload.re_pack(), 're-packing')
+    __run_processing(
+        uploads, parallel, lambda upload: upload.re_pack(), 're-packing',
+        wait_for_tasks=False)
+
+
+@uploads.command(help='Prepares files for being used in the upcoming NOMAD v1.0.')
+@click.argument('UPLOADS', nargs=-1)
+@click.option('--dry', is_flag=True, help='Just check, do nothing.')
+@click.option('-f', '--force', is_flag=True, help='Ignore warnings and perform the operation regardless.')
+@click.option('-q', '--quiet', is_flag=True, help='No output (only logs).')
+@click.option('--upload-ids', is_flag=True, help='Print uploads with errors.')
+@click.option('--label', type=str, help='A label to label log entries with.')
+@click.pass_context
+def prepare_migration(ctx, uploads, dry, force, label, quiet, upload_ids):
+    '''
+    Removes one of the raw files, either public or restricted depending on the embargo.
+    Files that need to be removed are saved as `quarantined` in the upload folder.
+    Only works on published uploads.
+    '''
+    import os.path
+    import os
+
+    logger = utils.get_logger(__name__)
+    if label:
+        logger = logger.bind(label=label)
+
+    _, uploads = query_uploads(ctx, uploads)
+    for upload in uploads:
+
+        def log_event(event: str, error: bool = False, **kwargs):
+            if not quiet:
+                print(f'    {"!!! " if error else ""}{event}', *[value for value in kwargs.values()])
+            method = getattr(logger, 'error' if error else 'info')
+            method(event, upload_id=upload.upload_id, **kwargs)
+
+        if not quiet:
+            print(f'Preparing {upload.upload_id} for migration ...')
+
+        if not upload.published:
+            log_event('upload is not published, nothing to do')
+            break
+
+        with_embargo_values: typing.List[bool] = []
+        for with_embargo_value in [True, False]:
+            search_request = search.SearchRequest().search_parameters(
+                upload_id=upload.upload_id, with_embargo=with_embargo_value)
+            if search_request.execute()['total'] > 0:
+                with_embargo_values.append(with_embargo_value)
+
+        if len(with_embargo_values) > 1:
+            if upload_ids:
+                print(upload.upload_id)
+            log_event('inconsistent upload', error=True)
+            break
+
+        if len(with_embargo_values) == 0:
+            if upload_ids:
+                print(upload.upload_id)
+            log_event('upload with no indexed entries', error=True)
+            break
+
+        with_embargo = with_embargo_values[0]
+
+        upload_files = files.PublicUploadFiles(
+            upload.upload_id, is_authorized=lambda *args, **kwargs: True, create=False)
+
+        obsolute_access = 'public' if with_embargo else 'restricted'
+        access = 'restricted' if with_embargo else 'public'
+        to_move = upload_files._raw_file_object(obsolute_access)
+        to_stay = upload_files._raw_file_object(access)
+
+        if not to_move.exists():
+            log_event('obsolute raw.zip was already removed', path=to_move.os_path)
+
+        elif to_stay.size < to_move.size and not force:
+            if upload_ids:
+                print(upload.upload_id)
+            log_event('likely inconsistent pack', error=True)
+
+        elif to_move.size == 22:
+            if not dry:
+                to_move.delete()
+            log_event('removed empty zip', path=to_move.os_path)
+
+        elif with_embargo and not force:
+            if upload_ids:
+                print(upload.upload_id)
+            log_event('embargo upload with non empty public file', error=True)
+
+        else:
+            if not dry:
+                target = upload_files._raw_file_object('quarantined')
+                assert not target.exists()
+                os.rename(to_move.os_path, target.os_path)
+            log_event('quarantined file', path=to_move.os_path)
+
+
+@uploads.command(help='Moves certain files from public or restricted to quarantine in published uploads.')
+@click.argument('UPLOADS', nargs=-1)
+@click.option(
+    '--file-pattern', type=str, multiple=True,
+    help=('The files as .zip patterns, e.g. "*/POTCAR". Default is all possible POTCAR patterns.'))
+@click.option('--dry', is_flag=True, help='Just check, do nothing.')
+@click.pass_context
+def quarantine_raw_files(ctx, uploads, dry, file_pattern):
+    import os.path
+    import os
+    import subprocess
+
+    if len(file_pattern) == 0:
+        file_pattern = [
+            '*/POTCAR', '*/POTCAR.gz', '*/POTCAR.xz', 'POTCAR', 'POTCAR.gz', 'POTCAR.xz']
+
+    sh_script = os.path.abspath('ops/scripts/quarantine-raw-files.sh')
+    cwd = os.path.abspath(os.curdir)
+
+    _, uploads = query_uploads(ctx, uploads)
+    for upload in uploads:
+        print(f'Moving {" ".join(file_pattern)} to quarantine in {upload.upload_id} ...')
+
+        if not upload.published:
+            print('   upload is not published, nothing to do')
+            break
+
+        upload_files = files.PublicUploadFiles(
+            upload.upload_id, is_authorized=lambda *args, **kwargs: True, create=False)
+
+        try:
+            os.chdir(upload_files.os_path)
+            p = subprocess.Popen(
+                ['sh', sh_script] + list(file_pattern), stdout=PIPE, stderr=PIPE)
+            _, err = p.communicate()
+
+            if p.returncode > 0:
+                print(f'   !!! could not move files: script has error output {err} !!!')
+        except Exception as e:
+            print(f'   !!! could not move files: {e} !!!')
+        finally:
+            os.chdir(cwd)
 
 
 @uploads.command(help='Attempt to abort the processing of uploads.')
@@ -390,3 +584,57 @@ def stop(ctx, uploads, calcs: bool, kill: bool, no_celery: bool):
     stop_all(proc.Calc.objects(running_query))
     if not calcs:
         stop_all(proc.Upload.objects(running_query))
+
+
+@uploads.group(help='Check certain integrity criteria')
+@click.pass_context
+def integrity(ctx):
+    pass
+
+
+@integrity.command(help='Uploads that have datasets with DOIs that do not exist.')
+@click.argument('UPLOADS', nargs=-1)
+@click.pass_context
+def dois(ctx, uploads):
+    import sys
+
+    from nomad.processing import Calc
+    from nomad.datamodel import Dataset
+    from nomad.doi import DOI
+    from nomad.search import SearchRequest
+
+    _, uploads = query_uploads(ctx, uploads)
+
+    for upload in uploads:
+        dataset_ids = Calc._get_collection().distinct(
+            'metadata.datasets',
+            dict(upload_id=upload.upload_id))
+
+        for dataset_id in dataset_ids:
+            dataset: Dataset = Dataset.m_def.a_mongo.objects(dataset_id=dataset_id).first()
+            if dataset is None:
+                print(f'ERROR: dataset does not exist {dataset_id}, seein in upload {upload.upload_id}', file=sys.stderr)
+                print(upload.upload_id)
+                continue
+
+            if dataset.doi is not None:
+                doi = DOI.objects(doi=dataset.doi).first()
+                if doi is None:
+                    continue
+
+                results = SearchRequest() \
+                    .search_parameters(upload_id=upload.upload_id, dataset_id=dataset_id) \
+                    .include('datasets') \
+                    .execute_paginated(per_page=1)
+
+                if results['total'] == 0:
+                    print(f'WARNING: dataset {dataset_id} not in index for upload {upload.upload_id}', file=sys.stderr)
+                    print(upload.upload_id)
+                    continue
+
+                if not any([
+                        dataset.get('doi') == doi.doi
+                        for dataset in results['results'][0]['datasets']]):
+
+                    print(f'WARNING: DOI of dataset {dataset_id} not in index for upload {upload.upload_id}', file=sys.stderr)
+                    print(upload.upload_id)
