@@ -1,16 +1,20 @@
-# Copyright 2018 Markus Scheidgen
 #
-# Licensed under the Apache License, Version 2.0 (the 'License');
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD. See https://nomad-lab.eu for further info.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an'AS IS' BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 
 from typing import List
 from abc import abstractmethod
@@ -123,12 +127,12 @@ class MethodNormalizer():
         atom_positions = sec_sys['atom_positions']
         geom_dict['atom_positions'] = np.array2string(
             atom_positions.to(ureg.angstrom).magnitude,  # convert to Angstrom
-            formatter={'float_kind': lambda x: "%.6f" % x},
+            formatter={'float_kind': lambda x: "%.6f" % x},  # type: ignore
         ).replace('\n', '')
         cell = sec_sys['lattice_vectors']
         geom_dict['simulation_cell'] = np.array2string(
             cell.to(ureg.angstrom).magnitude,  # convert to Angstrom
-            formatter={'float_kind': lambda x: "%.6f" % x},
+            formatter={'float_kind': lambda x: "%.6f" % x},  # type: ignore
         ).replace('\n', '')
         param_dict['settings_geometry'] = geom_dict
 
