@@ -133,6 +133,7 @@ class Calc(Proc):
             ('upload_id', 'mainfile'),
             ('upload_id', 'parser'),
             ('upload_id', 'tasks_status'),
+            ('upload_id', 'current_task'),
             ('upload_id', 'process_status'),
             ('upload_id', 'metadata.nomad_version'),
             'metadata.processed',
@@ -156,6 +157,11 @@ class Calc(Proc):
     def get(cls, id):
         return cls.get_by_id(id, 'calc_id')
 
+    @property
+    def entry_id(self) -> str:
+        ''' Just an alias for calc_id. '''
+        return self.calc_id
+
     @property
     def mainfile_file(self) -> PathObject:
         return self.upload_files.raw_file_object(self.mainfile)
@@ -578,21 +584,14 @@ class Calc(Proc):
         # metadata file name defined in nomad.config nomad_metadata.yaml/json
         # which can be placed in the directory containing the mainfile or somewhere up
         # highest priority is directory with mainfile
-
         metadata_file = config.metadata_file_name
         metadata_dir = os.path.dirname(self.mainfile_file.os_path)
+        upload_raw_dir = self.upload_files._raw_dir.os_path
 
         metadata = {}
+        metadata_part = None
+        # apply the nomad files of the current directory and parent directories
         while True:
-            # top-level nomad file can also contain an entries dict with entry
-            # metadata per mainfile as key
-            if metadata_dir == self.upload_files.os_path:
-                entries = metadata_part.get('entries', {})
-                metadata_part = entries.get(self.mainfile, {})
-                for key, val in metadata_part.items():
-                    metadata.setdefault(key, val)
-
-            # consider the nomad file of the current directory
             metadata_part = self.upload.metadata_file_cached(
                 os.path.join(metadata_dir, metadata_file))
             for key, val in metadata_part.items():
@@ -600,16 +599,23 @@ class Calc(Proc):
                     continue
                 metadata.setdefault(key, val)
 
-            if metadata_dir == self.upload_files.os_path:
+            if metadata_dir == upload_raw_dir:
                 break
 
             metadata_dir = os.path.dirname(metadata_dir)
 
+        # Top-level nomad file can also contain an entries dict with entry
+        # metadata per mainfile as key. This takes precedence of the other files.
+        entries = metadata_part.get('entries', {})
+        metadata_part = entries.get(self.mainfile, {})
+        for key, val in metadata_part.items():
+            metadata[key] = val
+
         if len(metadata) > 0:
             logger.info('Apply user metadata from nomad.yaml/json file')
 
         for key, val in metadata.items():
-            if key == 'entries':
+            if key in ['entries', 'skip_matching']:
                 continue
 
             definition = _editable_metadata.get(key, None)
@@ -642,9 +648,12 @@ class Calc(Proc):
         self._entry_metadata.processed = True
 
         if self.upload.publish_directly:
-            self._entry_metadata.published |= True
+            self._entry_metadata.published = False
 
-        self._read_metadata_from_file(logger)
+        try:
+            self._read_metadata_from_file(logger)
+        except Exception as e:
+            logger.error('could not process user metadata in nomad.yaml/json file', exc_info=e)
 
         # persist the calc metadata
         with utils.timer(logger, 'calc metadata saved'):
@@ -755,7 +764,7 @@ class Upload(Proc):
     meta: Any = {
         'strict': False,
         'indexes': [
-            'user_id', 'tasks_status', 'process_status', 'published', 'upload_time'
+            'user_id', 'tasks_status', 'process_status', 'published', 'upload_time', 'create_time'
         ]
     }
 
@@ -1050,6 +1059,30 @@ class Upload(Proc):
 
         self._continue_with('parse_all')
         try:
+            if config.reprocess_match:
+                with utils.timer(logger, 'calcs match on reprocess'):
+                    with_embargo_values = Calc._get_collection().distinct(
+                        'metadata.with_embargo', dict(upload_id=self.upload_id))
+                    with_embargo = with_embargo_values != [False]
+
+                    for filename, parser in self.match_mainfiles():
+                        calc_id = staging_upload_files.calc_id(filename)
+                        try:
+                            Calc.get(calc_id)
+                        except KeyError:
+                            calc = Calc.create(
+                                calc_id=calc_id,
+                                mainfile=filename, parser=parser.name,
+                                worker_hostname=self.worker_hostname,
+                                create_time=self.upload_time,
+                                upload_id=self.upload_id)
+
+                            calc.metadata.update(
+                                published=self.published,
+                                with_embargo=with_embargo)
+
+                            calc.save()
+
             with utils.timer(logger, 'calcs resetted'):
                 # check if a calc is already/still processing
                 processing = Calc.objects(
@@ -1089,16 +1122,7 @@ class Upload(Proc):
     def re_pack(self):
         ''' A *process* that repacks the raw and archive data based on the current embargo data. '''
         assert self.published
-
-        # mock the steps of actual processing
-        self._continue_with('uploading')
-        self._continue_with('extracting')
-        self._continue_with('parse_all')
-        self._continue_with('cleanup')
-
         self.upload_files.re_pack(self.user_metadata())
-        self.joined = True
-        self._complete()
 
     @process
     def process_upload(self):
@@ -1108,7 +1132,7 @@ class Upload(Proc):
         if self.from_oasis:
             # we might need to add datasets from the oasis before processing and
             # adding the entries
-            oasis_metadata_file = os.path.join(self.upload_files.os_path, 'raw', config.metadata_file_name + '.json')
+            oasis_metadata_file = os.path.join(self.upload_files._raw_dir.os_path, config.metadata_file_name + '.json')
             with open(oasis_metadata_file, 'rt') as f:
                 oasis_metadata = json.load(f)
             oasis_datasets = oasis_metadata.get('oasis_datasets', {})
@@ -1156,9 +1180,8 @@ class Upload(Proc):
         return self._upload_files
 
     @property
-    def staging_upload_files(self) -> ArchiveBasedStagingUploadFiles:
-        assert not self.published
-        return cast(ArchiveBasedStagingUploadFiles, self.upload_files)
+    def staging_upload_files(self) -> StagingUploadFiles:
+        return self.upload_files.to_staging_upload_files()
 
     @task
     def extracting(self):
@@ -1207,7 +1230,7 @@ class Upload(Proc):
                 stripped_f.write('Stripped POTCAR file. Checksum of original file (sha224): %s\n' % checksum)
             os.system(
                 '''
-                    awk < %s >> %s '
+                    awk < '%s' >> '%s' '
                     BEGIN { dump=1 }
                     /End of Dataset/ { dump=1 }
                     dump==1 { print }
@@ -1224,10 +1247,19 @@ class Upload(Proc):
         Returns:
             Tuples of mainfile, filename, and parsers
         '''
+        metadata = self.metadata_file_cached(
+            os.path.join(self.upload_files.os_path, 'raw', config.metadata_file_name))
+        skip_matching = metadata.get('skip_matching', False)
+        entries_metadata = metadata.get('entries', {})
+
         directories_with_match: Dict[str, str] = dict()
-        upload_files = self.staging_upload_files
+        upload_files = self.upload_files.to_staging_upload_files()
         for filename in upload_files.raw_file_manifest():
             self._preprocess_files(filename)
+
+            if skip_matching and filename not in entries_metadata:
+                continue
+
             try:
                 parser = match_parser(upload_files.raw_file_object(filename).os_path)
                 if parser is not None:
@@ -1362,9 +1394,23 @@ class Upload(Proc):
                 os.path.join(self.upload_files.os_path, 'raw', config.metadata_file_name))
 
             with self.entries_metadata(self.metadata) as calcs:
+                with utils.timer(logger, 'upload metadata updated'):
+                    def create_update(calc):
+                        calc.published = True
+                        calc.with_embargo = calc.with_embargo if calc.with_embargo is not None else False
+                        return UpdateOne(
+                            {'_id': calc.calc_id},
+                            {'$set': {'metadata': calc.m_to_dict(
+                                include_defaults=True, categories=[datamodel.MongoMetadata])}})
+
+                    Calc._get_collection().bulk_write([create_update(calc) for calc in calcs])
+
                 with utils.timer(logger, 'upload staging files packed'):
                     self.upload_files.pack(calcs)
 
+                with utils.timer(logger, 'index updated'):
+                    search.publish(calcs)
+
             with utils.timer(logger, 'upload staging files deleted'):
                 self.upload_files.delete()
 
@@ -1384,7 +1430,7 @@ class Upload(Proc):
     def _cleanup_after_re_processing(self):
         logger = self.get_logger(upload_size=self.upload_files.size)
         if self.published:
-            staging_upload_files = self.upload_files.to_staging_upload_files()
+            staging_upload_files = self.staging_upload_files
             logger.info('started to repack re-processed upload')
 
             with utils.timer(logger, 'staged upload files re-packed'):
@@ -1448,7 +1494,12 @@ class Upload(Proc):
             order_by: the property to order by
         '''
         query = Calc.objects(upload_id=self.upload_id)[start:end]
-        return query.order_by(order_by) if order_by is not None else query
+        if not order_by:
+            return query
+        if type(order_by) == str:
+            return query.order_by(order_by)
+        assert type(order_by) == tuple, 'order_by must be a string or a tuple if set'
+        return query.order_by(*order_by)
 
     @property
     def outdated_calcs(self):
